<?php

if (!defined('SMF'))
	die('Hacking attempt...');

/*

File: Subs-SPortal2.php
Version: Simple Portal 2.1 for SMF 2.x

getBlockInfo()
getFunctionInfo()
getCategoryInfo()
getArticleInfo()
createArticle()
fixCategoryArticles()
fixColumnRows()
changeBlockRow()
changeState()
spAllowedTo()
sp_loadPermissions()
sp_loadMemberGroups()
loadDatabaseTables()
databaseTableExists()

*/

// 	This function, returns all of the information about particular blocks.
function getBlockInfo($column_id = null, $block_id = null, $state = null, $show = null)
{
	global $context, $smcFunc, $settings, $txt;

	//	Setup some different methods of narrowing down the number of blocks that we need to return.
	$query = array();
	if(!empty($column_id))
		$query[] = 'col = {int:col}';
	if(!empty($block_id))
		$query[] = 'id_block = {int:id_block}';
	if(!empty($state))
		$query[] = 'state = {int:state}';
	
	//	Get the blocks.
	$request = $smcFunc['db_query']('','
		SELECT id_block, label, type, content, parameters, col, row, state, force_view, allowed_groups, permission_type, display
		FROM {db_prefix}sp_blocks' .
		(!empty($query) ? ' WHERE ' . implode(' AND ', $query) : '') .
		' ORDER BY row',
		array(
			'col' => !empty($column_id) ? $column_id : 0,
			'id_block' => !empty($block_id) ? $block_id : 0,
			'state' => 1,
		)
	);
	
	//	Go through each row.
	$return = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		//	Is this block even going to be shown in this section?
		if(!empty($show) && !getShowInfo($row['id_block'], $row['display']))
			continue;

		//	Are you even allowed to view this block?
		if(!spAllowedTo('block', $row['id_block']))
			continue;

		//	Format the block data in a nice and 'easy-to-use' way. 
		$return[] = array(
			'id' => $row['id_block'],
			'label' => $row['label'],
			'type' => $row['type'],
			'type_text' => !empty($txt['sp_function_'.$row['type'].'_label']) ? $txt['sp_function_'.$row['type'].'_label'] : $txt['sp_function_unkown_label'],
			'content' => $row['content'],
			'parameters' => $row['parameters'],
			'column' => $row['col'],
			'row' => $row['row'],
			'state' => empty($row['state']) ? 0 : 1,
			'force_view' => $row['force_view'],
			'allowed_groups' => $row['allowed_groups'],
			'permission_type' => $row['permission_type'],
			'display' => $row['display'],
		);
	}
	$smcFunc['db_free_result']($request);

	return $return;
}

//	This function, gets all of the necessary information about a particular sp block function.
function getFunctionInfo($function = null)
{
	global $smcFunc;

	$request = $smcFunc['db_query']('','
		SELECT id_function, name, parameter
		FROM {db_prefix}sp_functions' .
		(!empty($function) ? ' WHERE name = {string:function}' : '') .
		' ORDER BY id_function ASC',
		array(
			'function' => $function,
		)
	);
	$return = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		//There is one function who is not allowed to use by non admins
		if($row['name'] == 'sp_php' && !allowedTo('admin_forum'))
			continue;
		$return[] = array(
			'id' => $row['id_function'],
			'function' => $row['name'],
			'parameter' => $row['parameter'],
		);
	}
	$smcFunc['db_free_result']($request);

	return $return;
}

//	This function, gets all of the relevant information about sp categories.
function getCategoryInfo($category_id = null)
{
	global $scripturl, $context, $smcFunc, $settings, $txt;

	$request = $smcFunc['db_query']('','
		SELECT id_category, name, picture, articles, publish
		FROM {db_prefix}sp_categories' .
		(!empty($category_id) ? ' WHERE id_category = {int:category_id}' : '') .
		' ORDER BY id_category',
		array(
			'category_id' => $category_id,
		)
	);
	$return = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{

		$return[] = array(
			'id' => $row['id_category'],
			'name' => $row['name'],
			'picture' => array (
				'href' => $row['picture'],
				'image' => '<img src="' . $row['picture'] . '" alt="' . $row['name'] . '" width="75" />',
			),
			'articles' => $row['articles'],
			'publish' => $row['publish'],
		);
	}
	$smcFunc['db_free_result']($request);

	return $return;
}

//	This function returns article information.
function getArticleInfo($article_id = null, $category_id = null, $message_id = null, $approved = null)
{
	global $smcFunc;

	$query = array();
	if(!empty($article_id))
		$query[] = "id_article = {int:article_id}";
	if(!empty($category_id))
		$query[] = "id_category = {int:category_id}";
	if(!empty($message_id))
		$query[] = "id_message = {int:message_id}";
	if(!empty($approved))
		$query[] = "approved = '1'";

	$request = $smcFunc['db_query']('','
		SELECT a.id_article, a.id_category, a.id_message, a.approved, c.name
		FROM {db_prefix}sp_articles as a
			LEFT JOIN {db_prefix}sp_categories AS c ON (c.id_category = a.id_category)' .
		(!empty($query) ? ' WHERE ' . implode(' AND ', $query) : '') .
		' ORDER BY id_article',
		array(
			'article_id' => $article_id,
			'category_id' => $category_id,
			'message_id' => $message_id,
		)
	);
	$return = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{

		$return[] = array(
			'article' => array(
				'id' => $row['id_article'],
				'approved' => $row['approved'],
			),
			'category' => array(
				'id' => $row['id_category'],
				'name' => $row['name'],
			),
			'message' => array(
				'id' => $row['id_message'],
			),
		);
	}
	$smcFunc['db_free_result']($request);

	return $return;
}

//	Function to create a new article.
function createArticle($articleOptions)
{
	global $smcFunc;

	$articleOptions['id_category'] = !empty($articleOptions['id_category']) ? (int) $articleOptions['id_category'] : 0;
	$articleOptions['id_message'] = !empty($articleOptions['id_message']) ? (int) $articleOptions['id_message'] : 0;
	$articleOptions['approved'] = !empty($articleOptions['approved']) ? (int) $articleOptions['approved'] : 0;
		
	$request = $smcFunc['db_query']('','
		SELECT id_message
		FROM {db_prefix}sp_articles
		WHERE id_message = {int:id_message}',
		array(
			'id_message' => $articleOptions['id_message'],
		)
	);
	list ($exists) = $smcFunc['db_fetch_row']($request);
	$smcFunc['db_free_result']($request);

	if(empty($articleOptions['id_category']) || empty($articleOptions['id_message']) || $exists)
		return false;
		
	$smcFunc['db_insert']('normal', '{db_prefix}sp_articles',
		//	Columns to insert.
		array('id_category' => 'int', 'id_message' => 'int', 'approved' => 'int'),
		//	Data to put in.
		array($articleOptions['id_category'], $articleOptions['id_message'], $articleOptions['approved']),
		//	We had better tell SMF about the key, even though I can't remember why? ;)
		array('id_article')
	);

	$smcFunc['db_query']('','
		UPDATE {db_prefix}sp_categories
		SET articles = articles + 1
		WHERE id_category = {int:id_category}',
		array(
			'id_category' => $articleOptions['id_category'],
		)
	);
}

//	Function to fix the article counts for each category.
function fixCategoryArticles()
{
	global $smcFunc;

	//	Get a list of categories
	$categoryList = getCategoryInfo();
	$categoryIds = array();	

	//	Create a list of category ids.
	foreach($categoryList as $category)
		$categoryIds[] = $category['id'];

	//	Go through each category.
	foreach($categoryIds as $category) 
	{
		//	Firstly, get the current actual article count of the category.
		$article_count = 0;
		
		$request = $smcFunc['db_query']('','
			SELECT COUNT(*)
			FROM {db_prefix}sp_articles
			WHERE id_category = {int:id_category}',
			array(
				'id_category' => $category,
			)
		);
		list ($article_count) = $smcFunc['db_fetch_row']($request);
		$smcFunc['db_free_result']($request);
		
		//	Now, update the category so that it has the correct article count.
		$smcFunc['db_query']('','
			UPDATE {db_prefix}sp_categories
			SET articles = {int:article_count}
			WHERE id_category = {int:id_category}',
			array(
				'article_count' => $article_count,
				'id_category' => $category,
			)
		);
	}
}

//	Function to fix the columns for every block in a specific column.
function fixColumnRows($column_id = null)
{
	global $smcFunc;

	$blockList = getBlockInfo($column_id);
	$blockIds = array();	

	foreach($blockList as $block)
		$blockIds[] = $block['id'];

	$counter = 0;

	foreach($blockIds as $block) 
	{
		$counter = $counter + 1;
	
		$smcFunc['db_query']('','
			UPDATE {db_prefix}sp_blocks
			SET row = {int:counter}
			WHERE id_block = {int:block}',
			array(
				'counter' => $counter,
				'block' => $block,
			)
		);
	}
}

//	Function to change the row that a block is in.
function changeBlockRow($block_id = null, $direction = null)
{
	global $smcFunc;

	$blockInfo = getBlockInfo(null, $block_id);
	$blockInfo = $blockInfo[0];

	$smcFunc['db_query']('','
		UPDATE {db_prefix}sp_blocks
		SET row = {int:row}
		WHERE row = {int:row} ' . ($direction == 'up' ? '-' : '+') . ' 1
			AND col = {int:col}', 
		array(
			'row' => $blockInfo['row'],
			'col' => $blockInfo['column'],
		)
	);

	$smcFunc['db_query']('','
		UPDATE {db_prefix}sp_blocks
		SET row = row ' . ($direction == 'up' ? '-' : '+') . ' 1
		WHERE id_block = {int:block_id}
			AND col = {int:col}',
		array(
			'block_id' => $block_id,
			'col' => $blockInfo['column'],
		)
	);
}

//	Function to change the 'state' that an article, block or category has.
function changeState($type = null, $id = null)
{
	global $smcFunc;

	if($type == 'block')
		$query = array(
			'column' => 'state',
			'table' => 'blocks',
			'query_id' => 'id_block',
			'id' => $id
		);
	elseif($type == 'category')
		$query = array(
			'column' => 'publish',
			'table' => 'categories',
			'query_id' => 'id_category',
			'id' => $id
		);
	elseif($type == 'article')
		$query = array(
			'column' => 'approved',
			'table' => 'articles',
			'query_id' => 'id_article',
			'id' => $id
		);
	else
		return false;
	
	$request = $smcFunc['db_query']('','
		SELECT {raw:column}
		FROM {db_prefix}sp_{raw:table}
		WHERE {raw:query_id} = {int:id}',
		$query
	);
	
	list ($state) = $smcFunc['db_fetch_row']($request);
	$smcFunc['db_free_result']($request);
	
	$state = (int) $state;
	$state = ($state == 1) ? 0 : 1 ;
	
	$smcFunc['db_query']('','
		UPDATE {db_prefix}sp_{raw:table}
		SET {raw:column} = {int:state}
		WHERE {raw:query_id} = {int:id}',
		array(
			'table' => $query['table'],
			'column' => $query['column'],
			'state' => $state,
			'query_id' => $query['query_id'],
			'id' => $id,
		)
	);
}

// Make the allowed check faster, because i can cache this data!
// force_view is used for the way to make 3 diffrent ways to check blocks!
function sp_loadPermissions($type = null)
{
	global $sp_context, $smcFunc, $user_info;
	
	//Checkup
	if(empty($type) || !in_array($type, array('block', 'article'))) 
		return false;

	//Prevent double load :)	
	if(!isset($sp_context['Permissions'][$type]))
		$sp_context['Permissions'][$type] = array();
	else
		return !empty($sp_context['Permissions'][$type]); //On false allowed to nothing ;P
	
	//Blockquery Informations :)
	$where = '';

	if($type == 'block')
	{
		$query = array(
			'column' => 'allowed_groups',
			'table' => 'blocks',
			'field' => 'id_block',
			'permission_type' => ', permission_type',
		);
		$where = 'WHERE (permission_type = 2 
			OR (FIND_IN_SET(' . implode(', {raw:column}) OR FIND_IN_SET(', $user_info['groups']) . ', {raw:column})))';
	}
	else
	{
		$query = array(
			'column' => 'allowed_groups',
			'table' => 'articles',
			'field' => 'is_article',
			'permission_type' => '',
		);
		$where = 'WHERE (FIND_IN_SET(' . implode(', {raw:column}) OR FIND_IN_SET(', $user_info['groups']) . ', {raw:column}))';
	}

	
	//	Get the necessary block information.
	$request = $smcFunc['db_query']('','
		SELECT {raw:column} AS allowed_groups, {raw:field} AS id {raw:permission_type}
		FROM {db_prefix}sp_{raw:table}
		'.$where,
		$query
	);
	//The output will only show the allowed positions ;D
	while($row = $smcFunc['db_fetch_assoc']($request))
	{
		//This Permission need a extra checkup 8), it's a faster query now, a lot faster!
		if(!empty($row['permission_type']) && $row['permission_type'] == 1)
		{
			if(count(array_intersect($user_info['groups'], explode(',', $row['allowed_groups']))) != count($user_info['groups']))
				continue;
		}
		$sp_context['Permissions'][$type][$row['id']] = $row['id'];
	}
	$smcFunc['db_free_result']($request);
	
	//Okay chached it ;) now it should be faster... a lot faster 8)
	return !empty($sp_context['Permissions'][$type]);
}

//	Function to check if users are allowed to view certain blocks.
function spAllowedTo($type = null, $id = null)
{
	global $sp_context;

	// SP moderators are supermen. :D i need to do nothing if this correct! It's to late checked!
	// And this is the reason why admin unselect never work!
	// If we like to fix this we need to think about a way to allowed it in a other way or insert a force admin check
	if(allowedTo('sp_moderate'))
		return true;

	//	Sanitize the ID.
	$id = (int) $id;

	// Make sure that we have a proper id, load Permissions and store a chache 
	if(empty($id) || sp_loadPermissions($type) === false || empty($type))
		return false;

	//Pah if it isset than you allowed, if not than not :)
	return isset($sp_context['Permissions'][$type][$id]);
}

/*
	void sp_loadMemberGroups(Array $selectedGroups = array, Array $removeGroups = array(), string $show = 'normal', string $contextName = 'member_groups')
	This will file the $context['member_groups'] to the given options
	$selectedGroups means all groups who should be shown as selcted, if you like to check all than insert an 'all'
		You can also Give the function a string with '2,3,4'
	$removeGroups this group id should not shown in the list
	$show have follow options
		'normal' => will show all groups, and add a guest and regular member (Standard)
		'post' => will load only post groups
		'master' => will load only not postbased groups
	$contextName where the datas should stored in the $context;

	The Administrator group id 1 will evertime ignored!
*/
function sp_loadMemberGroups($selectedGroups = array(), $show = 'normal', $contextName = 'member_groups', $subContext = 'SPortal') 
{
	global $context, $smcFunc, $txt;
	
	//Some additional Language stings are needed
	loadLanguage('ManageBoards');

	// Make sure its empty
	if(!empty($subContext))
		$context[$subContext][$contextName] = array();
	else
		$context[$contextName] = array();
	
	//Preseting some things :)
	if(!is_array($selectedGroups))
		$checked = strtolower($selectedGroups) == 'all';
	else
		$checked = false;
	
	if(!$checked && !empty($selectedGroups)) {
		if(!is_array($selectedGroups)) 
			$selectedGroups = explode(',', $selectedGroups);
		//Remove all strings, i will only allowe ids :P
		foreach($selectedGroups as $k => $i)
			$selectedGroups[$k] = (int) $i;
		$selectedGroups = array_unique($selectedGroups);
	}
	else
		$selectedGroups = array();

	//Okay let's checkup the show function
	$show_option = array(
		'normal' => 'id_group != 1',
		'post' => 'min_posts != -1',
		'master' => 'min_posts = -1 AND id_group != 1',
	);

	$show = strtolower($show);

	if(!isset($show_option[$show]))
		$show = 'normal';

	// Guest and Members are added manually. Only on normal ond master View =)
	if($show == 'normal' || $show == 'master') {
		$context[$contextName][-1] = array(
			'id' => -1,
			'name' => $txt['membergroups_guests'],
			'checked' => $checked || in_array(-1, $selectedGroups),
			'is_post_group' => false,
		);
		$context[$contextName][0] = array(
			'id' => 0,
			'name' => $txt['membergroups_members'],
			'checked' => $checked || in_array(0, $selectedGroups),
			'is_post_group' => false,
		);
	}
	
	// Load membergroups.
	$request = $smcFunc['db_query']('', '
		SELECT group_name, id_group, min_posts
		FROM {db_prefix}membergroups
		WHERE {raw:show}
		ORDER BY min_posts, id_group != {int:global_moderator}, group_name',
		array(
			'show' => $show_option[$show],
			'global_moderator' => 2,
		)
	);
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		$context[$contextName][(int) $row['id_group']] = array(
			'id' => $row['id_group'],
			'name' => trim($row['group_name']),
			'checked' => $checked || in_array($row['id_group'], $selectedGroups),
			'is_post_group' => $row['min_posts'] != -1,
		);
	}
	$smcFunc['db_free_result']($request);
}

function sp_queryFix($tourniquet)
{
	global $sportal_version, $context;

	if (SMF == 'SSI' || empty($context['template_layers']) || WIRELESS)
		return $tourniquet;

	$fix = str_replace('{version}', $sportal_version, '<a href="http://www.simpleportal.net/" target="_blank">SimplePortal {version}</a>');

	$tourniquet = preg_replace(base64_decode('figsIFNpbXBsZSBNYWNoaW5lcyBMTEM8L2E+KX4='), base64_decode('LCBTaW1wbGUgTWFjaGluZXMgTExDPC9hPjxiciAvPg==') . $fix, $tourniquet);

	if (strpos($tourniquet, 'www.simpleportal.net') === false)
	{
		$fix = '<div class="smalltext" style="text-align: center; width: 100%;">' . $fix . '</div>';
		$tourniquet = preg_replace('~</body>\s*</html>~', $fix, $tourniquet);
	}

	return $tourniquet;
}

// Function to get a block's display/show information.
function getShowInfo($block_id = null, $display = null)
{
	global $smcFunc, $context, $modSettings;

	// Do we have the display info?
	if(empty($display))
	{
		// Make sure that its an integer.
		$block_id = (int) $block_id;

		// We need an ID.
		if(empty($block_id))
			return false;

		// Get the info.
		$result = $smcFunc['db_query']('','
			SELECT display
			FROM {db_prefix}sp_blocks
			WHERE id_block = {int:id_block}
			LIMIT 1',
			array(
				'id_block' => $block_id,
			)
		);
		list ($display) = $smcFunc['db_fetch_row']($result);
		$smcFunc['db_free_result']($result);
	}

	// Some variables for ease.
	$action = !empty($context['current_action']) ? $context['current_action'] : '';
	$board = !empty($context['current_board']) ? $context['current_board'] : '';
	$portal = (empty($action) && empty($board)) ? true : false;

	// Set some action exceptions.
	$exceptions = array(
		'post' => array('announce', 'editpoll', 'emailuser', 'post2', 'sendtopic'),
		'register' => array('activate', 'coppa'),
		'forum' => array('collapse'),
		'admin' => array('credits', 'theme', 'viewquery', 'viewsmfile'),
		'moderate' => array('groups'),
		'login' => array('reminder'),
		'profile' => array('trackip', 'viewprofile'),
	);

	// Set the action to more known one.
	foreach($exceptions as $key => $exception)
	{
		if(in_array($action, $exception))
			$action = $key;
	}

	// If no display info and/or integration disabled and we are on portal; show it!
	if((empty($display) || empty($modSettings['sp_enableIntegration'])) && $portal)
		return true;
	// No display info and/or integration disabled and no portal; no need...	
	elseif(empty($display) || empty($modSettings['sp_enableIntegration']))
		return false;
	// Get ready for real action if you haven't yet.
	elseif(!is_array($display))
		$display = explode(',', $display);

	// Did we disable all blocks for this action?
	if(!empty($modSettings['sp_' . $action . 'IntegrationHide']))
		return false;
	// If we will display show the block.
	elseif(in_array('all', $display))
		return true;
	// If we are on portal, show portal blocks; if we are on forum, show forum blocks.
	elseif(($portal && (in_array('portal', $display) || in_array('sportal', $display))) || (!$portal && in_array('sforum', $display)))
		return true;
	// If current action or board is in display info; or we should display block in all actions or all boards then you can display block.
	elseif((!empty($action) && (in_array('allaction', $display) || in_array($action, $display))) || (!empty($board) && (in_array('allboard', $display) || in_array($board, $display))))
		return true;

	// Ummm, no block!
	return false;
}

/*
This is a simple function that return nothing if the language file exist and english if it not exists
This will help to make it possible to load each time the english language!
*/
function sp_languageSelect($template_name) 
{
	global $user_info, $language, $settings, $context;
	global $sourcedir;
	static $already_loaded = array();

	if(isset($already_loaded[$template_name]))
		return $already_loaded[$template_name];

	$lang = isset($user_info['language']) ? $user_info['language'] : $language;

	// Make sure we have $settings - if not we're in trouble and need to find it!
	if (empty($settings['default_theme_dir']))
	{
		require_once($sourcedir . '/ScheduledTasks.php');
		loadEssentialThemeData();
	}
	
	// For each file open it up and write it out!
	$allTemplatesExists = array();
	foreach (explode('+', $template_name) as $template)
	{
		// Obviously, the current theme is most important to check.
		$attempts = array(
			array($settings['theme_dir'], $template, $lang, $settings['theme_url']),
			array($settings['theme_dir'], $template, $language, $settings['theme_url']),
		);

		// Do we have a base theme to worry about?
		if (isset($settings['base_theme_dir']))
		{
			$attempts[] = array($settings['base_theme_dir'], $template, $lang, $settings['base_theme_url']);
			$attempts[] = array($settings['base_theme_dir'], $template, $language, $settings['base_theme_url']);
		}

		// Fallback on the default theme if necessary.
		$attempts[] = array($settings['default_theme_dir'], $template, $lang, $settings['default_theme_url']);
		$attempts[] = array($settings['default_theme_dir'], $template, $language, $settings['default_theme_url']);

		// Try to find the language file.
		$allTemplatesExists[$template] = false;
		$already_loaded[$template] = 'english';
		foreach ($attempts as $k => $file) {
			if (file_exists($file[0] . '/languages/' . $file[1] . '.' . $file[2] . '.php'))
			{
				$already_loaded[$template] = '';
				$allTemplatesExists[$template] = true;
				break;
			}
		}
	}
	//So all need to be true that it work ;)
	foreach($allTemplatesExists as $exist)
		if(!$exist) {
			$already_loaded[$template_name] = 'english';
			return 'english';
		}

	//Everthing is fine, let's go back :D
	$already_loaded[$template_name] = '';
	return '';
}

function sp_checkSyntax($code)
{
    return @eval('return true;' . $code);
}

/*
	bool loadDatabaseTables ()
	- Preload database Tables this will be only done once per script call
	- Return false on error
	- Mysql 3.x+ Compatible ;) for SMF 2.0.x
*/
function loadDatabaseTables() 
{
	global $database_tables, $db_name, $smcFunc;
	
	if(isset($database_tables))
		return !empty($database_tables);
	else
		$database_tables = array();
			
	//Prepare a Checkup ;D
	$request = $smcFunc['db_query']('', '
		SHOW TABLES
		FROM `{raw:db_name}`',
		array(
			'db_name' => $db_name,
		)
	);
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$database_tables[] = strtolower(current($row));
	$smcFunc['db_free_result']($request);

	return !empty($database_tables);
}

/*
	bool databaseTableExists (string tablename, bool $addprefix = true)
	- Return true if the tablename exits
	- $addprefix will automatic add the $db_prefix (Check also it this is allready done!)
*/
function databaseTableExists($tablename = '', $addprefix = true) 
{
	global $database_tables, $db_prefix;
	
	//Nothing to Serach for ;)
	if(empty($tablename))
		return false;

	//Error ?_?
	if(empty($database_tables) && !loadDatabaseTables())
		return false;
	
	//First thing i do.. i know normal i should not so this ;P But it's good enouph for smf ;)
	$tablename = strtolower($tablename);
	
	//Table Prefix auto add? :D
	if($addprefix === true && strpos($tablename, $db_prefix) !== 0)
	{
		//Looks not so nice ;) but it work trust me ;P
		if(strpos($tablename, '{db_prefix}') !== false)
			$tablename = str_replace('{db_prefix}', $db_prefix, $tablename);
		elseif(strpos($tablename, '$db_prefix') !== false)
			$tablename = str_replace('$db_prefix', $db_prefix, $tablename);
		else
			$tablename = $db_prefix.$tablename;
	}

	return in_array($tablename, $database_tables);
}

//This function return the correct array that is called.
function sp_loadCalendarData($type, $low_date, $high_date = false) {
	global $sourcedir;
	static $loaded;
	if(!isset($loaded)) {
		if(file_exists($sourcedir . '/Subs-Calendar.php')) {
			require_once($sourcedir . '/Subs-Calendar.php');
			$loaded['getEvents'] = 'getEventRange';
			$loaded['getBirthdays'] = 'getBirthdayRange';
			$loaded['getHolidays'] = 'getHolidayRange';
		}
		else {
			require_once($sourcedir . '/Calendar.php');
			$loaded['getEvents'] = 'calendarEventArray';
			$loaded['getBirthdays'] = 'calendarBirthdayArray';
			$loaded['getHolidays'] = 'calendarHolidayArray';
		}	
	}
	return !empty($loaded[$type]) ? $loaded[$type]($low_date, ($high_date === false ? $low_date : $high_date)) : array();
}

function sp_startElement($parser, $name, $attrs)
{
	global $rss_data, $rss_current, $rss_main;

	switch($name)
	{
		case 'RSS':
		case 'RDF:RDF':
		case 'ITEMS':
			$rss_current = '';
			break;
		case 'CHANNEL':
			$rss_main = 'CHANNEL';
			break;
		case 'IMAGE':
			$rss_main = 'IMAGE';
			$rss_data['IMAGE'] = array();
			break;
		case 'ITEM':
			$rss_main = 'ITEMS';
			break;
		default:
			$rss_current = $name;
			break;
	}
}

function sp_endElement($parser, $name)
{
	global $rss_data, $rss_current, $rss_counter;

	$rss_current = '';

	if ($name == 'ITEM')
		$rss_counter++;
}

function sp_characterData($parser, $data)
{
	global $rss_data, $rss_current, $rss_main, $rss_counter;

	if (!empty($rss_current))
	{
		switch($rss_main)
		{
			case 'CHANNEL':
				if (isset($rss_data[$rss_current]))
					$rss_data[$rss_current] .= $data;
				else
					$rss_data[$rss_current] = $data;
				break;
			case 'IMAGE':
				if (isset($rss_data[$rss_main][$rss_current]))
					$rss_data[$rss_main][$rss_current] .= $data;
				else
					$rss_data[$rss_main][$rss_current] = $data;
				break;
			case 'ITEMS':
				if (isset($rss_data[$rss_main][$rss_counter][$rss_current]))
					$rss_data[$rss_main][$rss_counter][$rss_current] .= $data;
				else
					$rss_data[$rss_main][$rss_counter][$rss_current] = $data;
				break;
		}
	}
}

?>